/*
 * GNOME Contacts
 */

ContactsListPane.frame:dir(ltr) {
  border-width: 0 1px 0 0;
}

ContactsListPane.frame:dir(rtl) {
  border-width: 0 0 0 1px;
}

.contacts-map {
    background-color: @theme_bg_color;
}

/* contatcs view new color */
.contacts-view {
  background-color: transparent;
}

/* Background color in contacts pane, similar to .documents-main-view.view */
.contacts-main-view.view {
  background-color: mix(@theme_bg_color, @theme_base_color, 0.2);
}

.contacts-suggestion {
  background-color: darker(@theme_bg_color);
}

/* Border on the right in the left menu toolbar */
.contacts-left-header-bar:dir(ltr) {
 border-right-width: 1px;
}

.contacts-left-header-bar:dir(rtl) {
 border-left-width: 1px;
}

.contacts-left-header-bar:dir(ltr),
.contacts-right-header-bar:dir(rtl) {
  border-top-right-radius: 0;
}

.contacts-right-header-bar:dir(ltr),
.contacts-left-header-bar:dir(rtl) {
  border-top-left-radius: 0;
}

.contacts-avatar-frame.frame {
 border-width: 1px 1px 1px 1px;
 border-style: solid;
 border-color: @borders;
 border-image: none;
 border-radius: 0;
 padding: 0;
}

.main-avatar-frame.frame {
 border-width: 1px;
 border-style: solid;
 border-color: @borders;
 border-radius: 6px;
}

/* Primary toolbar with no line at top to avoid conflicts with frame border */
ContactsWindow .primary-toolbar.toolbar {
 border-width: 0;
 border-bottom-width: 1px;
 border-style: solid;
 border-color: @borders;
}

.contacts-entry {
 box-shadow: none;
 border-image: none;
 border-width: 1px;
 border-radius: 4px;
 border-style: solid;
 border-color: @borders;
}

.contacts-entry:selected,
.contacts-entry:selected:focus {
 background-color: @theme_selected_bg_color;
 color: @theme_selected_fg_color;
}

.contacts-entry.contacts-postal-entry {
 border-radius: 0 0 0 0;
 border-width: 1px 1px 0 1px;
}

.contacts-entry.contacts-postal-entry:nth-child(first) {
 border-radius: 4px 4px 0 0;
}

.contacts-entry.contacts-postal-entry:nth-child(last) {
 border-radius: 0 0 4px 4px;
 border-width: 1px;
}

.button.contacts-square {
 padding: 0px;
}

.contacts-combo .button {
 border-image: none;
 border-width: 1px;
 border-style: solid;
 border-color: @borders;
 background-image: none;
 background-color: @theme_base_color;
}

.contacts-watermark {
  color: #bebebe;
  text-shadow: 1px 1px alpha(@theme_base_color, 0.6);
}
